<html py:extends="master.xhtml" py:strip="True">
  <head py:block="head" py:strip="True">
    <title py:block="master_title">Change Password</title>
  </head>

  <body py:block="body" py:strip="True">
    <div class="row">
      <div class="col-md-12">
        ${new_password_form.display(value=form_data, action=action)}
      </div>
    </div>
  </body>
</html>
